library view_model;

import 'package:flutter/widgets.dart';

class ViewModelProvider extends InheritedWidget {
  final ViewModelFactory factory;

  ViewModelProvider({this.factory, Widget child}): super(child: child);

  @override
  bool updateShouldNotify(ViewModelProvider oldWidget) => factory != oldWidget.factory;

  dynamic provide<T extends ViewModel>() => factory.crateViewModel<T>();

  static ViewModelProvider of(BuildContext context) =>
      context.inheritFromWidgetOfExactType(ViewModelProvider);
}

abstract class ViewModelFactory {
  dynamic crateViewModel<T extends ViewModel>();
}

abstract class ViewModel {
  static dynamic of(BuildContext context) {
    var iVM = context.inheritFromWidgetOfExactType(InheritedViewModel) as InheritedViewModel;
    return iVM.viewModel;
  }

  void dispose() {}
}

class InheritedViewModel extends InheritedWidget {
  final dynamic viewModel;

  InheritedViewModel({this.viewModel, Widget child}): super(child: child);

  @override
  bool updateShouldNotify(InheritedViewModel oldWidget) =>
      viewModel != oldWidget.viewModel;

}

abstract class Page extends StatefulWidget {
  @override
  State createState() => initPageState();

  dynamic initPageState<T extends PageState>();
}

abstract class PageState<T extends Page, VT extends ViewModel> extends State<T> {
  VT viewModel;

  @override
  Widget build(BuildContext context) {
    viewModel = ViewModelProvider.of(context).provide<VT>();
    return InheritedViewModel(
      viewModel: viewModel,
      child: buildWidget(context),
    );
  }

  Widget buildWidget(BuildContext context);
}
